#!/usr/bin/python3

"""Pytest fixtures to simplify tests"""

import pytest


@pytest.fixture
def cachedir(tmp_path):
    """Create temporary dir named 'cachedir'"""
    cache_dir = tmp_path / "cache"
    cache_dir.mkdir()
    return cache_dir


@pytest.fixture
def outdir(tmp_path):
    """Create temporary dir named 'outdir'"""
    out_dir = tmp_path / "out"
    out_dir.mkdir()
    return out_dir


@pytest.fixture
def musicdir(tmp_path):
    """Create temporary dir named 'musicdir'"""
    music_dir = tmp_path / "music"
    music_dir.mkdir()
    return music_dir



#!/usr/bin/python3

"""Test the functions in metalfinder.output"""

import os
import datetime

import metalfinder.output as mfo


CONCERT_LIST = [{'artist_id': '1',
                 'datetime': '2022-11-06T20:00:00',
                 'description': 'Concert 1',
                 'id': '12345',
                 'lineup': ['Arch Enemy'],
                 'title': 'Concert 1',
                 'updated': datetime.datetime(2022, 6, 6, 20, 5, 13, 366428),
                 'url': 'https://www.bandsintown.com/e/12345',
                 'venue': {'city': 'Oslo',
                 'name': 'TheFooBar'}},
                {'artist_id': '2',
                 'datetime': '2022-11-07T20:00:00',
                 'description': 'Concert 2',
                 'id': '54321',
                 'lineup': ['Napalm Death', 'Amorphis'],
                 'title': 'Concert 2',
                 'updated': datetime.datetime(2022, 6, 5, 20, 5, 13, 366428),
                 'url': 'https://www.bandsintown.com/e/54321',
                 'venue': {'city': 'Montreal',
                 'name': 'MTelus'}}]


def test_pretty_strings():
    """Test function pretty_strings()"""
    concert = {'id': '1',
               'lineup': ['Arch Enemy'],
               'datetime': '2022-01-01T20:00:00',
               'venue': {'name': 'MTelus',
                         'city': 'Montreal'},
               'description': 'This is a description'}
    formatted_tuple = ('1', 'Arch Enemy', '2022-01-01 20:00', 'MTelus, Montreal',
                       'This is a description', 'https://www.bandsintown.com/e/1')
    assert formatted_tuple == mfo.pretty_strings(concert)


def test_atom(outdir):
    """Test function atom()"""
    output = mfo.atom(CONCERT_LIST)
    # We need to write to a file, since this object is a
    # feedgenerator.django.utils.feedgenerator.Atom1Feed object
    with open(str(outdir) + '.atom', 'w', encoding='utf-8') as final:
        output.write(final, 'utf-8')
    with open('tests/test_files/concerts.atom', 'r', encoding='utf-8') as file1:
        valid_atom = file1.read()
        with open(str(outdir) + '.atom', 'r', encoding='utf-8') as file2:
            testfile_atom = file2.read()
            assert valid_atom == testfile_atom


def test_txt():
    """Test function txt()"""
    output = mfo.txt(CONCERT_LIST)
    with open('tests/test_files/concerts.txt', 'r', encoding='utf-8') as file1:
        valid_txt = file1.read()
        assert valid_txt == output


def test_json():
    """Test function _json()"""
    output = mfo._json(CONCERT_LIST)  # pylint: disable=W0212
    with open('tests/test_files/concerts.json', 'r', encoding='utf-8') as file1:
        valid_json = file1.read()
        assert valid_json == output


def test_output_wrapper(outdir):
    """Test wrapper function output_wrapper()"""
    output_path = os.path.join(outdir, 'foo.txt')
    mfo.output_wrapper(CONCERT_LIST, output_path)
    with open('tests/test_files/concerts.txt', 'r', encoding='utf-8') as file1:
        valid_txt = file1.read()
        with open(output_path, 'r', encoding='utf-8') as file2:
            output_file = file2.read()
        assert valid_txt == output_file



#!/usr/bin/python3

# Copyright 2015-2018 by Chris Forrette
# Copyright 2022 by Louis-Philippe Véronneau
#
# This file was originally part of python-bandsintown.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Bandintown API provider
"""

try:
    from urllib.parse import quote, urljoin
except ImportError:
    from urllib import quote
    from urlparse import urljoin

import logging
import sys
import requests

from packaging import version

if version.parse(requests.__version__) < version.parse("2.27"):
    try:
        import simplejson
    except ImportError:
        sys.exit("Your version of requests is older than 2.27. You either need"
                  " to update it to a version >=2.27 or to install simplejson.")


class BandsintownError(Exception):
    """Base class for other exceptions"""
    def __init__(self, message, response=None):
        super().__init__()
        self.message = message
        self.response = response


    def __str__(self):
        return self.message


class BandsintownInvalidAppIdError(BandsintownError):
    """Raised whenever a request is made with an invalid app id"""


class Client():
    """Main class for the Bandsintown provider"""
    api_base_url = 'https://rest.bandsintown.com'


    def __init__(self, app_id):
        """
        Args:
            app_id: Required app id, can be any string
        """
        self.app_id = app_id
        self.default_params = {'app_id': self.app_id}


    def request(self, path, params = None):
        """
        Executes a request to the Bandsintown API and returns the response
        object from `requests`

        Args:
            path: The API path to append to the base API URL for the request
            params: Optional dict to tack on query string parameters to request

        Returns:
            Response object from `requests`
        """
        url = urljoin(self.api_base_url, path)
        request_params = self.default_params.copy()
        if params is not None:
            request_params.update(params)
        response = requests.get(
            url,
            headers={'Accept': 'application/json'},
            params=request_params
        )
        # TODO: Some responses aren't always valid JSON? This probably happens
        # because we're not passing the right info? To debug.
        if version.parse(requests.__version__) < version.parse("2.27"):
            try:
                data = response.json()
            except simplejson.errors.JSONDecodeError:  # requests <2.27
                data = []
        else:
            try:
                data = response.json()
            except requests.exceptions.JSONDecodeError: # pylint: disable=E0701  # requests >=2.27
                data = []

        invalid_appid_message = ('User is not authorized to access this resource '
                                 'with an explicit deny')
        if 'Message' in data and data['Message'] == invalid_appid_message:
            message = 'Invalid Bandsintown App ID (API key)'
            raise BandsintownInvalidAppIdError(message, response)

        missing_appid_message = 'Missing Authentication Token'
        if 'message' in data and data['message'] == missing_appid_message:
            message = 'Missing Bandsintown App ID (API key)'
            raise BandsintownInvalidAppIdError(message, response)
        return data


    def artists_events(self, artistname, date=None):
        """
        Searches for events for a single artist, with an optional date range,
        using this endpoint:

            https://app.swaggerhub.com/apis/Bandsintown/PublicAPI/3.0.0#/upcoming_artist_events/artistEvents

        Args:
            artistname: Artist name to search for
            date: Optional date string filter, can be a specific date in the
            format: "yyyy-mm-dd", a range "yyyy-mm-dd,yyyy-mm-dd", or can be a
            few keyword values like "upcoming" or "all"


        Returns:
            A list of event data, which could be empty, None if artist not
            found, raises `BandsintownError` for other unknown error

        Usage:
            client = Client(app_id=1234)
            client.artists_events('Bad Religion')
            client.artists_events('Bad Religion', date='2018-02-01,2018-02-28')
        """
        params = {}

        if date:
            params['date'] = date

        data = self.request(f'artists/{quote(artistname)}/events', params)

        if 'errorMessage' in data:
            if data['errorMessage'] == '[NotFound] The artist was not found':
                logging.info("Artist '%s' could not be found on Bandsintown, "
                             "skipping it.", artistname)
                return None
            raise BandsintownError('Unknown error with request', data)

        return data



#!/usr/bin/python3

"""Test the functions in metalfinder.concerts"""

import logging
import os
import pickle

from collections import namedtuple

import pytest

import metalfinder.concerts as mfc


def test_has_changed_false():
    """Test has_changed() is False for existing concert"""
    concert = {'id': '102459446', 'title': 'Placeholder'}
    concert_cache = [{'artist_id': '862',
                      'id': '102459446'}]
    concert_is_new = mfc.has_changed(concert_cache, concert)
    assert concert_is_new is False


def test_has_changed_true(caplog):
    """Test has_changed() is True for new concert"""
    concert = {'id': '12345', 'title': 'Placeholder'}
    concert_cache = [{'artist_id': '862',
                      'id': '102459446'}]
    caplog.set_level(logging.INFO)
    concert_is_new = mfc.has_changed(concert_cache, concert)
    assert concert_is_new is True
    for record in caplog.records:
        assert record.levelname == 'INFO'
        assert record.message == 'Concert 12345 is new'


def test_write_concert_cache(cachedir):
    """Test function write_concert_cache()"""
    data = ['1', '2']
    mfc.write_concert_cache(data, str(cachedir))
    concert_cache_file = os.path.join(str(cachedir), 'concert_cache')
    with open(concert_cache_file, 'rb') as _cache:
        concert_cache = pickle.load(_cache)
        assert concert_cache == data


def test_get_concert_cache(cachedir):
    """Test function get_concert_cache()"""
    data = ['1', '2']
    concert_cache_file = os.path.join(str(cachedir), 'concert_cache')
    with open(concert_cache_file, 'wb') as _cache:
        pickle.dump(data, _cache)
    assert data == mfc.get_concert_cache(str(cachedir))


@pytest.mark.parametrize("artist_list, split_artist_list",
    [(['Mark+Spencer', 'Terry/ Berry', 'John &James'],
      ['Mark', 'Spencer', 'Terry', 'Berry', 'John', 'James']),
     (['Napalm Death', 'Arch Enemy'], ['Napalm Death', 'Arch Enemy'])])
def test_query_bit(artist_list, split_artist_list, monkeypatch):
    """Test function query_bit(). We mostly want to test that artists are split properly."""
    def mocked_client(bit_appid):
        return bit_appid
    def mocked_bit_request(artist, concert_list, *args): # pylint: disable=W0613
        concert_list.append(artist)
        return concert_list
    monkeypatch.setattr('metalfinder.api.bandsintown.Client', mocked_client)
    monkeypatch.setattr('metalfinder.concerts.bit_request', mocked_bit_request)
    Args = namedtuple('Args', ['bit_appid', 'max_date'])
    mocked_args = Args('placeholder', 'placeholder')
    assert split_artist_list == mfc.query_bit(artist_list, mocked_args)


def test_filter_location():
    """Test function filter_location()"""
    concert_list = [{'artist_id': '862',
                     'id': '102459446',
                     'venue': {'city': 'Montreal',
                               'other': 'Foobar'}},
                    {'artist_id': '1',
                     'id': '12345',
                     'venue': {'city': 'Oslo',
                               'other': 'Foobar'}}]
    filtered_list = [{'artist_id': '1',
                      'id': '12345',
                      'venue': {'city': 'Oslo',
                                'other': 'Foobar'}}]
    assert filtered_list == mfc.filter_location(concert_list, 'Oslo')



#!/usr/bin/python3

"""Unit tests for metalfinder.scan"""

import os
import pickle
import logging
import shutil

import pytest

import metalfinder.scan as mfs


NO_ARTIST_ERROR = "<class 'KeyError'>: 'artist'"
NO_HEADER_ERROR = "<class 'mutagen.flac.FLACNoHeaderError'>"


def move_testfile(filename, musicdir):
    """Move a test file from our local test_files dir to a temporary dir
       musicdir', so that the path is reproducible"""
    origfile = os.path.join('tests/test_files', filename)
    destfile = os.path.join(str(musicdir), filename)
    shutil.copyfile(origfile, destfile)
    return destfile


def change_mtime(artist, filename, musicdir):
    """Change the mtime of a test file, so that it's reproducible"""
    destfile = move_testfile(filename, musicdir)
    mtime = int('1111111111')
    os.utime(destfile, (mtime, mtime))
    good_song_cache = {os.path.join(str(musicdir), filename):
            [1111111111.0, artist]}
    return destfile, good_song_cache


@pytest.mark.parametrize("artist, filename",
    [('Arch Enemy', 'arch_enemy.flac'),
     ('Napalm Death', 'napalm_death.flac')])
def test_has_changed_new(artist, filename, musicdir):
    """Test if has_changed() picks up a new song"""
    new_song_cache = {}
    song_cache = {'/home/foo/NOFX/a.flac': [1449768136.0, 'NOFX'],
                  '/home/foo/NOFX/b.flac': [1449768136.0, 'NOFX']}
    destfile, good_song_cache = change_mtime(artist, filename, musicdir)
    new_song_cache, file_changed = mfs.has_changed(song_cache, new_song_cache, destfile)
    assert new_song_cache == good_song_cache
    assert file_changed is True


@pytest.mark.parametrize("artist, filename",
    [('Arch Enemy', 'arch_enemy.flac'),
     ('Napalm Death', 'napalm_death.flac')])
def test_has_changed_old(artist, filename, musicdir):
    """Test if has_changed() skips over a previously cached file"""
    new_song_cache = {}
    destfile, song_cache = change_mtime(artist, filename, musicdir)
    new_song_cache, file_changed = mfs.has_changed(song_cache, new_song_cache, destfile)
    assert new_song_cache == song_cache
    assert file_changed is False


@pytest.mark.parametrize("artist, filepath",
    [('Arch Enemy', 'tests/test_files/arch_enemy.flac'),
     ('Napalm Death', 'tests/test_files/napalm_death.flac')])
def test_get_artist_ok(artist, filepath):
    """Test get_artist() extracts the tag properly"""
    assert artist == mfs.get_artist(None, filepath)


@pytest.mark.parametrize("filepath, error",
    [('tests/test_files/no_artist.flac', NO_ARTIST_ERROR),
     ('tests/test_files/no_header.flac', NO_HEADER_ERROR)])
def test_get_artist_common_errors(filepath, error, caplog):
    """Test get_artist() returns nothing when there is no artist tag"""
    caplog.set_level(logging.WARNING)
    assert mfs.get_artist(None, filepath) is None
    for record in caplog.records:
        assert error in record.args[1]


def test_write_song_cache(cachedir):
    """Test function write_song_cache()"""
    data = ['1', '2']
    mfs.write_song_cache(data, str(cachedir))
    song_cache_file = os.path.join(str(cachedir), 'song_cache')
    with open(song_cache_file, 'rb') as _cache:
        song_cache = pickle.load(_cache)
        assert song_cache == data


def test_get_song_cache(cachedir):
    """Test function get_song_cache()"""
    data = ['1', '2']
    song_cache_file = os.path.join(str(cachedir), 'song_cache')
    with open(song_cache_file, 'wb') as _cache:
        pickle.dump(data, _cache)
    assert data == mfs.get_song_cache(str(cachedir))


def test_write_artist_cache(cachedir):
    """Test function write_artist_cache()"""
    data = ['1', '2']
    mfs.write_artist_cache(data, str(cachedir))
    artist_cache_file = os.path.join(str(cachedir), 'artist_cache')
    with open(artist_cache_file, 'r', encoding='utf-8') as _cache:
        artist_cache = _cache.read()
        assert artist_cache == '1\n2'


def test_get_artist_cache(cachedir):
    """Test function get_artist_cache()"""
    data = '1\n2'
    artist_cache_file = os.path.join(str(cachedir), 'artist_cache')
    with open(artist_cache_file, 'w', encoding='utf-8') as _cache:
        _cache.write(data)
    assert ['1', '2'] == mfs.get_artist_cache(str(cachedir))


@pytest.mark.parametrize("artist, filename",
    [('Arch Enemy', 'arch_enemy.flac'),
     ('Napalm Death', 'napalm_death.flac')])
def test_scan_dir(artist, filename, musicdir):
    """Test function scan_dir()"""
    _, good_song_cache = change_mtime(artist, filename, musicdir)
    artist_list, new_song_cache = mfs.scan_dir(musicdir, '', [])
    assert artist_list == {artist}
    assert new_song_cache == good_song_cache


@pytest.mark.parametrize("artist, filename",
    [('Arch Enemy', 'arch_enemy.flac'),
     ('Napalm Death', 'napalm_death.flac')])
def test_scan_wrapper(artist, filename, cachedir, musicdir):
    """Test wrapper function scan_wrapper()"""
    _ = move_testfile(filename, musicdir)
    artist_list = mfs.scan_wrapper(str(musicdir), str(cachedir))
    assert artist_list == {artist}


def test_issue21(cachedir, tmpdir):
    """Test that we don't crash on broken symlinks"""
    musicdir = tmpdir.join("music")
    musicdir.mkdir()
    musicdir.join("brokensymlink.mp3").mksymlinkto("nonexistent")
    # we don't actually need the result here
    _ = mfs.scan_wrapper(musicdir, cachedir)


def test_issue22(cachedir, musicdir):
    """Test artists are removed from the cache when their files are removed"""
    for filename in ['arch_enemy.flac', 'napalm_death.flac']:
        destfile = move_testfile(filename, musicdir)
    artist_list = mfs.scan_wrapper(str(musicdir), str(cachedir))
    assert artist_list == {'Arch Enemy', 'Napalm Death'}
    os.remove(destfile)
    artist_list = mfs.scan_wrapper(str(musicdir), str(cachedir))
    assert artist_list == {'Arch Enemy'}


def test_issue28(cachedir, musicdir, caplog):
    """Test metalfinder continues running if a file passed through get_artist()
       raises an exception"""
    caplog.set_level(logging.WARNING)
    for filename in ['no_header.flac', 'no_artist.flac', 'arch_enemy.flac']:
        _ = move_testfile(filename, musicdir)
    artist_list = mfs.scan_wrapper(str(musicdir), str(cachedir))
    assert artist_list == {'Arch Enemy'}
    assert NO_ARTIST_ERROR in caplog.records[0].args[1]
    assert NO_HEADER_ERROR in caplog.records[1].args[1]



#!/usr/bin/python3

"""Test the functions in metalfinder.cli"""

import logging
import os

from unittest import mock
from datetime import date, timedelta
from pathlib import Path

import pytest

from metalfinder.cli import output_choices, date_lint, dir_lint, parse_args
from metalfinder import __version__


def test_output_choices_ok(outdir):
    """Test output_choices() when path is valid"""
    choices = { 'text': str(outdir) + '/test.txt',
                'json': str(outdir) + '/test.json',
                'atom': str(outdir) + '/test.atom' }
    for _, value in choices.items():
        assert output_choices(value) == value


def test_output_choices_invalid_extension():
    """Test output_choices() errors when file extension is invalid"""
    with pytest.raises(SystemExit):
        assert output_choices('foo.notvalid')


def test_output_choices_invalid_path():
    """Test output_choices() errors when path is invalid"""
    with pytest.raises(SystemExit):
        assert output_choices('/asdkaskdm/kmasdkmD/foo.txt')


# date_lint()
def test_date_lint_ok():
    """Test date_lint() for valid ISO 8601 input"""
    assert date_lint('2100-01-01') == '2100-01-01'


def test_date_lint_not_iso_8601():
    """Test date_lint() errors when input is not ISO 8601"""
    with pytest.raises(SystemExit):
        date_lint('21000101')


def test_date_lint_maxdate_too_early():
    """Test date_lint() errors when max_date earlier than today"""
    with pytest.raises(SystemExit):
        yesterday = date.today() - timedelta(days = 1)
        date_lint(str(yesterday))


def test_dir_not_path():
    """Test dir_lint() errors when dir is not a valid"""
    with pytest.raises(SystemExit):
        dir_lint('')


# parse_args()
def test_parser_no_dir(cachedir):
    """Test argparse without --directory"""
    args = parse_args(['-o', 'foo.txt', '-l', 'Montreal', '-b', 'FooBar',
                       '-c', str(cachedir)])
    assert args.directory is None
    assert args.output == 'foo.txt'
    assert args.location == 'Montreal'
    assert args.bit_appid == 'FooBar'
    assert args.cache_dir == str(cachedir)


def test_parser_no_output(cachedir, outdir):
    """Test argparse without --output"""
    with pytest.raises(SystemExit):
        _args = parse_args(['-d', str(outdir), '-l', 'Montreal', '-b', 'FooBar',
                            '-c', str(cachedir)])


def test_parser_no_location(cachedir, outdir):
    """Test argparse without --location"""
    with pytest.raises(SystemExit):
        _args = parse_args(['-d', str(outdir), '-l', 'Montreal', '-b', 'FooBar',
                            '-c', str(cachedir)])


@mock.patch.dict(os.environ, {'METALFINDER_BIT_APPID': 'FooBar'})
def test_parser_api_with_env(cachedir, outdir):
    """Test argparse with --bit_appid and METALFINDER_BIT_APPID"""
    args = parse_args(['-d', str(outdir), '-o', 'foo.txt', '-l', 'Montreal',
                       '-b', 'NonDefault', '-c', str(cachedir)])
    assert args.directory == str(outdir)
    assert args.output == 'foo.txt'
    assert args.location == 'Montreal'
    assert args.bit_appid == 'NonDefault'
    assert args.cache_dir == str(cachedir)


def test_parser_api_no_env(cachedir, outdir):
    """Test argparse with --bit_appid, without METALFINDER_BIT_APPID"""
    args = parse_args(['-d', str(outdir), '-o', 'foo.txt', '-l', 'Montreal',
                       '-b', 'FooBar', '-c', str(cachedir)])
    assert args.directory == str(outdir)
    assert args.output == 'foo.txt'
    assert args.location == 'Montreal'
    assert args.bit_appid == 'FooBar'
    assert args.cache_dir == str(cachedir)


@mock.patch.dict(os.environ, {'METALFINDER_BIT_APPID': 'FooBar'})
def test_parser_no_api_with_env(cachedir, outdir):
    """Test argparse without --bit_appid, with METALFINDER_BIT_APPID"""
    args = parse_args(['-d', str(outdir), '-o', 'foo.txt', '-l', 'Montreal',
                       '-c', str(cachedir)])
    assert args.directory == str(outdir)
    assert args.output == 'foo.txt'
    assert args.location == 'Montreal'
    assert args.bit_appid == 'FooBar'
    assert args.cache_dir == str(cachedir)


def test_parser_no_api_no_env(cachedir, outdir):
    """Test argparse without --bit_appid and METALFINDER_BIT_APPID"""
    with pytest.raises(SystemExit):
        _args = parse_args(['-d', str(outdir), '-l', 'Montreal', '-b', 'FooBar',
                            '-c', str(cachedir)])


def test_parse_cache_default(cachedir, outdir, monkeypatch):
    """Test argparse with default cache"""
    def mockreturn():
        return cachedir
    monkeypatch.setattr(Path, "home", mockreturn)
    args = parse_args(['-d', str(outdir), '-o', 'foo.txt', '-l', 'Montreal',
                       '-b', 'FooBar'])
    assert args.directory == str(outdir)
    assert args.output == 'foo.txt'
    assert args.location == 'Montreal'
    assert args.bit_appid == 'FooBar'
    assert args.cache_dir == str(cachedir) + '/.cache/metalfinder'


def test_parser_cache_non_default(cachedir, outdir):
    """Test argparse with cli non-default cache"""
    args = parse_args(['-d', str(outdir), '-o', 'foo.txt', '-l', 'Montreal',
                       '-b', 'FooBar', '-c', str(cachedir)])
    assert args.directory == str(outdir)
    assert args.output == 'foo.txt'
    assert args.location == 'Montreal'
    assert args.bit_appid == 'FooBar'
    assert args.cache_dir == str(cachedir)


def test_parser_verbose_default():
    """Test argpase with default log level"""
    _args = parse_args(['-o', 'foo.txt', '-l', 'Montreal', '-b', 'FooBar'])
    assert logging.getLogger().getEffectiveLevel() == logging.WARNING


def test_parser_verbose_debug():
    """Test argpase with info log level"""
    _args = parse_args(['-o', 'foo.txt', '-l', 'Montreal', '-b', 'FooBar',
                        '--verbose'])
    assert logging.root.getEffectiveLevel() == logging.INFO


# This test cannot be done using pytest.raises(Foo), as it will exit after
# catching the exception, thus not letting us test the captured stdout.
def test_parser_version(capsys):
    """Test argparse --version outputs the right version number"""
    try:
        _args = parse_args(['--version'])
    except SystemExit:
        out, _err = capsys.readouterr()
        assert out.rstrip() == "metalfinder " + __version__



#!/usr/bin/python3

"""
scan a music directory and output a list of artists
"""

import logging
import os
import pickle

import mutagen


def has_changed(song_cache, new_song_cache, fullpath):
    """Query song cache to know if file has changed (or is new) since last run.
       If it has, update the cache."""
    file_changed = False
    artist = None
    mtime = os.stat(fullpath).st_mtime
    if (fullpath not in song_cache) or (fullpath in song_cache and
                                        song_cache[fullpath][0] != mtime):
        file_changed = True
        artist = get_artist(artist, fullpath)
    if file_changed:
        new_song_cache[fullpath] = [mtime, artist]
    else:
        new_song_cache[fullpath] = [mtime, song_cache[fullpath][1]]
    return new_song_cache, file_changed


def get_artist(artist, fullpath):
    """Get the artist tag from a song"""
    try:
        artist = mutagen.File(fullpath, easy=True)["artist"]
        artist = ''.join(artist)  # list to string
    except Exception as _error: # pylint: disable=W0703
        error_message = f"{type(_error)}: {_error}"
        logging.warning("Could not scan file %s, skipping it: %s", fullpath, error_message)
    return artist


def write_song_cache(data, cache_dir):
    """Write song cache file to disk"""
    song_cache_file = os.path.join(cache_dir, 'song_cache')
    with open(song_cache_file, 'wb') as _cache:
        pickle.dump(data, _cache)


def get_song_cache(cache_dir):
    """Get the song cache if it exists"""
    song_cache = {}
    song_cache_file = os.path.join(cache_dir, 'song_cache')
    if os.path.isfile(song_cache_file):
        with open(song_cache_file, 'rb') as _cache:
            song_cache = pickle.load(_cache)
    return song_cache


def write_artist_cache(data, cache_dir):
    """Write artist cache file to disk"""
    artist_cache_file = os.path.join(cache_dir, 'artist_cache')
    with open(artist_cache_file, 'w+', encoding='utf-8') as _cache:
        _cache.write('\n'.join(data))


def get_artist_cache(cache_dir):
    """Get the artist cache if it exists"""
    artist_list = []
    artist_cache_file = os.path.join(cache_dir, 'artist_cache')
    if os.path.isfile(artist_cache_file):
        with open(artist_cache_file, 'r', encoding='utf-8') as _cache:
            artist_list = _cache.read().splitlines()
    return artist_list


def scan_dir(music_dir, song_cache, artist_list):
    """Scan a directory and output a list of artists"""
    new_song_cache = {}
    for dirname, _, filenames in os.walk(music_dir, topdown=False):
        for song in filenames:
            fullpath = os.path.abspath(os.path.join(dirname, song))
            if not song.endswith(('.flac', '.mp3', '.ogg')):
                continue
            try:
                new_song_cache, file_changed = has_changed(song_cache, new_song_cache, fullpath)
            except FileNotFoundError as _error:
                logging.warning("Could not load tags from file %s, skipping: %s", fullpath, _error)
                continue
            artist = new_song_cache[fullpath][1]
            if artist and file_changed:
                artist_list.append(artist)
    stale_song_cache = set(song_cache) - set(new_song_cache)
    for stale_song in stale_song_cache:
        artist = song_cache[stale_song][1]
        if artist in artist_list:
            artist_list.remove(artist)
    return set(artist_list), new_song_cache


def scan_wrapper(music_dir, cache_dir):
    """Wrapper function to manage the scan"""
    artist_list = get_artist_cache(cache_dir)
    if not music_dir:
        return artist_list
    song_cache = get_song_cache(cache_dir)
    artist_list, new_song_cache = scan_dir(music_dir, song_cache, artist_list)
    write_song_cache(new_song_cache, cache_dir)
    write_artist_cache(artist_list, cache_dir)
    return artist_list



#!/usr/bin/python3

# Copyright 2015-2018 by Chris Forrette
# Copyright 2022 by Louis-Philippe Véronneau
#
# This file was originally part of python-bandsintown.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Test the functions in metalfinder.api.bandsintown"""

from unittest import TestCase, mock

import metalfinder.api.bandsintown as mab


def test_init_stores_properties():
    """Test properties are stored properly"""
    app_id = 1234
    client = mab.Client(app_id)

    assert client.api_base_url == 'https://rest.bandsintown.com'
    assert client.app_id == app_id
    assert client.default_params == {'app_id': app_id}


@mock.patch('requests.get')
class RequestTestCase(TestCase):
    """Series of tests for Client.request()"""
    def test_builds_and_executes_request(self, mock_requests_get):
        """Test request() returns expected values"""
        mock_result = {'name': 'Bad Religion'}
        mock_response = mock.MagicMock()
        mock_response.json.return_value = mock_result
        mock_requests_get.return_value = mock_response

        app_id = 5678
        client = mab.Client(app_id)
        result = client.request('things')
        url = client.api_base_url + '/things'
        mock_requests_get.assert_called_with(
            url,
            headers={'Accept': 'application/json'},
            params=client.default_params
        )
        self.assertEqual(mock_result, result)

    def test_merges_passed_in_params(self, mock_requests_get): #pylint: disable=R0201
        """Test request() accepts extra parameters"""
        app_id = 5678
        date = '2018-01-01'
        client = mab.Client(app_id)
        client.request('more/things', {'date': date})
        url = client.api_base_url + '/more/things'
        mock_requests_get.assert_called_with(
            url,
            headers={'Accept': 'application/json'},
            params={
                'app_id': app_id,
                'date': date
            }
        )

    def test_raises_error_for_bad_api_key(self, mock_requests_get):
        """Test request() raises error when app ID is invalid"""
        invalid_appid_message = ('User is not authorized to access this '
                                 'resource with an explicit deny')
        mock_result = {'Message': invalid_appid_message}
        mock_response = mock.MagicMock()
        mock_response.json.return_value = mock_result
        mock_requests_get.return_value = mock_response

        client = mab.Client('')

        with self.assertRaises(mab.BandsintownInvalidAppIdError):
            client.request('stuff')


@mock.patch.object(mab.Client, 'request')
class EventsTestCase(TestCase):
    """Series of tests for Client.artist_events()"""
    def test_requests_artist_events(self, mock_requests_get):
        """Test artist_events() returns expected values"""
        data = [
            {
                'id': '20590797',
                'datetime': '2018-05-04T13:00:00'
            },
            {
                'id': '21752009',
                'datetime': '2018-05-05T19:00:00'
            }
        ]
        mock_requests_get.return_value = data

        client = mab.Client('my-app-id')
        artist = 'Every Time I Die'
        result = client.artists_events(artist)

        mock_requests_get.assert_called_with('artists/Every%20Time%20I%20Die/events', {})
        self.assertEqual(result, data)

    def test_takes_date_param(self, mock_requests_get):
        """Test artist_events() can accept a date range"""
        data = [{'id': '1234'}]
        mock_requests_get.return_value = data

        client = mab.Client('my-app-id')
        artist = 'Every Time I Die'
        date = '2018-01-15,2018-02-15'
        result = client.artists_events(artist, date)

        mock_requests_get.assert_called_with(
            'artists/Every%20Time%20I%20Die/events',
            {'date': date}
        )
        self.assertEqual(result, data)

    def test_returns_none_when_artist_not_found(self, mock_requests_get):
        """Test artist_events() returns None when no artist is found"""
        mock_requests_get.return_value = {'errorMessage': '[NotFound] The artist was not found'}
        client = mab.Client('my-app-id')
        self.assertIsNone(client.artists_events('foo'))

    def test_returns_none_when_no_events_found(self, mock_requests_get):
        """Test artist_events() returns None when no event is found"""
        mock_requests_get.return_value = []
        client = mab.Client('my-app-id')
        self.assertEqual(client.artists_events('bar'), [])



#!/usr/bin/python3

"""
get concerts data from API provider
"""

import datetime
import logging
import os
import pickle
import re

from .api.bandsintown import Client


def has_changed(concert_cache, concert):
    """Query concert cache to know if concert has changed (or is new) since
    last run."""
    concert_is_new = False
    if not any(concert['id'] in item['id'] for item in concert_cache):
        concert_is_new = True
        logging.info("Concert %s is new", concert['id'])
    return concert_is_new


def write_concert_cache(data, cache_dir):
    """Write concert cache file to disk"""
    concert_cache_file = os.path.join(cache_dir, 'concert_cache')
    with open(concert_cache_file, 'wb') as _cache:
        pickle.dump(data, _cache)


def get_concert_cache(cache_dir):
    """Get the concert cache if it exists"""
    concert_cache = []
    concert_cache_file = os.path.join(cache_dir, 'concert_cache')
    if os.path.isfile(concert_cache_file):
        with open(concert_cache_file, 'rb') as _cache:
            concert_cache = pickle.load(_cache)
    return concert_cache


def query_bit(artist_list, args):
    """Query Bandsintown for concert list"""
    bit_client = Client(args.bit_appid)
    concert_list = []
    split_pattern = re.compile(r"\+|/|&")
    for artist in artist_list:
        if re.search(split_pattern, artist):
            splitted_artist = re.split(split_pattern, artist)
            for s_artist in splitted_artist:
                concert_list = bit_request(s_artist.strip(), concert_list,
                                           args.max_date, bit_client)
        else:
            concert_list = bit_request(artist, concert_list,
                                       args.max_date, bit_client)
    return concert_list


def bit_request(artist, concert_list, max_date, bit_client):
    """Make API request to Bandsintown"""
    # TODO: Why are we getting some empty artists?
    if artist != '':
        if max_date:
            date_range = str(datetime.date.today()) + ',' + max_date
            concerts = bit_client.artists_events(artist, date=date_range)
        else:
            concerts = bit_client.artists_events(artist)
        if concerts is not None:
            concert_list = concert_list + concerts
    return concert_list


def filter_location(concert_list, location):
    """Filter concerts based on location"""
    concert_list = [concert for concert in concert_list if
            (concert['venue']['city'] == location)]
    return concert_list


def bit(artist_list, args):
    """Wrapper function for Bandsintown provider"""
    concert_cache = get_concert_cache(args.cache_dir)
    concert_list = query_bit(artist_list, args)
    concert_list = filter_location(concert_list, args.location)
    for concert in concert_list:
        if has_changed(concert_cache, concert):
            concert['updated'] = datetime.datetime.now()
            concert_cache.append(concert)
    write_concert_cache(concert_cache, args.cache_dir)
    return concert_cache



#!/usr/bin/python3

"""Main function wrapper"""

import sys

from .cli import parse_args
from .scan import scan_wrapper
from .concerts import bit
from .output import output_wrapper


def main(argv=sys.argv[1:]):  # pylint: disable=W0102
    """Main function."""
    args = parse_args(argv)
    artist_list = scan_wrapper(args.directory, args.cache_dir)
    concert_list = bit(artist_list, args)
    output_wrapper(concert_list, args.output)


if __name__ == "__main__":
    main()
